DAT fileName  byte "TouchButtonServo130716a", 0
CON
{{ ****** Public Notes ******
  This program uses the QuickStart's touchpads to control up to 12 servos.
  The 8 servos will use pins 8 through 15 (between the pad pins and
  the LED pins).  These are referred to as "static servos" because they
  do not continuously move.  Their position must be adjusted using
  pads 0 and 1.

  There are also four "oscillating servos" starting at pin 24 (after the
  LED pins).  The period of oscillation is controlled using pads 2 and 3.
  The period uses servo refresh cycles as units.  At a 50Hz refresh rate,
  an oscillation period of 100 will be two seconds.  Increasing the
  period will make the servos oscillate slower.

  The lower pulse length end point may be adjusted using pads 4 and 5.
  The upper end point may be adjusted using pads 6 and 7.

  The higher number pad always increases the parameter while the lower
  number pad decreases the parameter.

  The end points are used by both the static servos and the oscillating
  servos.

  The amount the parameter changes increases the longer the pad is pressed.

  The parameters are displayed in a terminal window and also using the
  QuickStart's LEDs.

  The static servos' position is displayed (with LEDs) in binary with
  the center position zero.  Pulse lengths shorter than 1500us are
  displayed with the least significant bit on the right.  Pulse
  length longer than 1500us are displayed with the least significant
  bit on the left.  All the LEDs will be off when the pulse length is
  1500us (centered).

  The end point parameters use the same system or least significant
  bits as the static servos position.

  The end point parameters and oscillation parameters are displayed with the
  LEDs only while they are being changed and for a short time afterward.
  The period parameter has the least significant bit on the right.
  
}}
{ ****** Private Notes ****** 
  The servo's start position will be the centered.  Use left four
  pads to increase the pulse length of the signal and the right
  four pads to decrease the pulse length of the signal.
  The LEDs will all be off at the center position.  At other
  positions they will display the distance from the center
  position.  Longer than centered pulse lengths will be displayed
  with the least significant bit on the far left LED.  Shorter
  pulse lengths will be display with the least significant bit
  on the far right LED.

  07b is working okay.
  10a try to have different modes for setting end points and oscillation speed.
  11a simplify interface.  No more "modes" (not that I got them to work).
  11a compiles.
  11b kind of works
  11e word pretty well.  The end points can only be change way from center.
  11f is working but there's a problem with the debug window.
  11g slower baud didn't help.
  11h Try Parallax Serial Terminal as the serial object.  That works much better.
  130716a Update object to use "Servo32v9".
}
CON

  _CLKMODE = XTAL1 + PLL16X
  _CLKFREQ = 80_000_000

  DEFAULT_MAX_PULSE = 2000
  DEFAULT_MIN_PULSE = 1000
  ABSOLUTE_MAX_PULSE = 3000
  ABSOLUTE_MIN_PULSE = 300
  DEFAULT_PERIOD = 100  ' 2 seconds
  ABSOLUTE_MAX_PERIOD = 3600 * DEFAULT_PERIOD ' two hours?
  ABSOLUTE_MIN_PERIOD = 16
 
  CENTER = 1500
  STATIC_SERVOS = 8       
  OSCILLATING_SERVOS = 4
  FIRST_OSCILLATING_SERVO_PIN = 24
 
  FIRST_SERVO_PIN = 8
  SMALL_CHANGE = 1
  LARGE_CHANGE = 16 * SMALL_CHANGE
  SMALL_ENDPOINT_CHANGE = 1
  LARGE_ENDPOINT_CHANGE = 4 * SMALL_ENDPOINT_CHANGE
  SMALL_OSCILLATION_CHANGE = 1
  LARGE_OSCILLATION_CHANGE = 4 * SMALL_OSCILLATION_CHANGE
  PRESS_THRESHOLD = 40
  HOLD_LED_THRESHOLD = 80
  DEBUG_BAUD = 115200 '57600 ' use 9600 or less when using Simple Serial and friends   

  ' LED mode enumeration
  #0, STATIC_MODE, OSCILLATION_MODE, ENDPOINT_LOW_MODE, ENDPOINT_HIGH_MODE

  DEFAULT_LED_MODE = STATIC_MODE
  
VAR

  long delay, ledShow, interval, debugStack[50]
  long globalTime, frameInterval
  
  long frameCount, servoPosition, oscillatingPosition, minPulse, maxPulse, period
  ' Keep these six variables together and in order.
  
  byte servoPin[STATIC_SERVOS], oscillatingServoPin[OSCILLATING_SERVOS]
  byte button[8], sideFlag, fastInputFlag, ledMode
  
OBJ

  Buttons : "TouchButtons111029a"
  Servo : "Servo32v9"
  Debug : "Parallax Serial Terminal"
  
PUB Main | change, localIndex, pseudoPercent, nextFrameTime, stateCount, oldState, currentState, fastInputCount, previousButton

  delay := clkfreq / 100
 
  
  ledShow := 0
  if maxPulse == 0
    minPulse := DEFAULT_MIN_PULSE
    maxPulse := DEFAULT_MAX_PULSE
    frameInterval := clkfreq / 50
    period := DEFAULT_PERIOD
  frameCount := 0
  waitcnt(clkfreq * 3 + cnt)                           
  
  cognew(DebugLoop, @debugStack)                 ' starts a cog 6 left
   
  Servo.Start                                    ' starts a cog 5 left
 
  waitcnt(clkfreq / 20 + cnt)

  servoPosition := CENTER        
   
  oscillatingPosition := DEFAULT_MIN_PULSE        

  Buttons.start(@delay, @button)            ' Launch the touch buttons driver sampling 100 times a second
  dira[23..16]~~                            ' Set the LEDs as outputs

  stateCount := 0
  currentState := -1
  fastInputFlag := 0
  fastInputCount := 0
  change := 0                                    
  globalTime := cnt
  nextFrameTime := cnt + frameInterval
  repeat
    oldState := currentState
    currentState := -1
    waitcnt(nextFrameTime)
    if button[0] == 1
      ledMode := STATIC_MODE
      if previousButton == 0
        if ++stateCount => PRESS_THRESHOLD
          if change < LARGE_CHANGE
            change++
          stateCount~
      else
        stateCount~
        previousButton := 0
        change := SMALL_CHANGE
      if servoPosition - change < minPulse
        servoPosition := minPulse
      else
        servoPosition -= change
    elseif button[1] == 1
      ledMode := STATIC_MODE
      if previousButton == 1
        if ++stateCount => PRESS_THRESHOLD
          if change < LARGE_CHANGE
            change++
          stateCount~
      else
        stateCount~
        previousButton := 1
        change := SMALL_CHANGE
      if servoPosition + change > maxPulse
        servoPosition := maxPulse
      else
        servoPosition += change
   

    elseif button[2] == 1
      ledMode := OSCILLATION_MODE
      if previousButton == 2
        if ++stateCount => PRESS_THRESHOLD
          if change < LARGE_OSCILLATION_CHANGE
            change++
          stateCount~
      else
        stateCount~
        previousButton := 2
        change := SMALL_OSCILLATION_CHANGE
      if period - change < ABSOLUTE_MIN_PERIOD
        period := ABSOLUTE_MIN_PERIOD
      else
        period -= change
    elseif button[3] == 1
      ledMode := OSCILLATION_MODE
      if previousButton == 3
        if ++stateCount => PRESS_THRESHOLD
          if change < LARGE_OSCILLATION_CHANGE
            change++
          stateCount~
      else
        stateCount~
        previousButton := 3
        change := SMALL_OSCILLATION_CHANGE
      if period + change > ABSOLUTE_MAX_PERIOD
        period := ABSOLUTE_MAX_PERIOD
      else
        period += change
    elseif button[4] == 1
      ledMode := ENDPOINT_LOW_MODE
      if previousButton == 4
        if ++stateCount => PRESS_THRESHOLD
          if change < LARGE_ENDPOINT_CHANGE
            change++
          stateCount~
      else
        stateCount~
        previousButton := 4
        change := SMALL_ENDPOINT_CHANGE
      if minPulse - change < ABSOLUTE_MIN_PULSE
        minPulse := ABSOLUTE_MIN_PULSE
      else
        minPulse -= change
    elseif button[5] == 1
      ledMode := ENDPOINT_LOW_MODE
      if previousButton == 5
        if ++stateCount => PRESS_THRESHOLD
          if change < LARGE_ENDPOINT_CHANGE
            change++
          stateCount~
      else
        stateCount~
        previousButton := 5
        change := SMALL_ENDPOINT_CHANGE
      if minPulse + change > maxPulse
        minPulse := maxPulse
      else
        minPulse += change
    elseif button[6] == 1
      ledMode := ENDPOINT_HIGH_MODE
      if previousButton == 6
        if ++stateCount => PRESS_THRESHOLD
          if change < LARGE_ENDPOINT_CHANGE
            change++
          stateCount~
      else
        stateCount~
        previousButton := 6
        change := SMALL_ENDPOINT_CHANGE
      if maxPulse - change < minPulse
        maxPulse := minPulse
      else
        maxPulse -= change
    elseif button[7] == 1
      ledMode := ENDPOINT_HIGH_MODE
      if previousButton == 7
        if ++stateCount => PRESS_THRESHOLD
          if change < LARGE_ENDPOINT_CHANGE
            change++
          stateCount~
      else
        stateCount~
        previousButton := 7
        change := SMALL_ENDPOINT_CHANGE
      if maxPulse + change > ABSOLUTE_MAX_PULSE
        maxPulse := maxPulse
      else
        maxPulse += change
    else
      if previousButton == -1
        if ++stateCount => HOLD_LED_THRESHOLD
          stateCount~
          ledMode := DEFAULT_LED_MODE
      else
        previousButton := -1
        stateCount~
      
       
    nextFrameTime += frameInterval
    pseudoPercent := ((frameCount * 100) / period )  ' How far into the cycle are we? 
    if pseudoPercent < 50
      oscillatingPosition := minPulse + ((2 * frameCount * (maxPulse - minPulse))/ period)

    else
      oscillatingPosition := maxPulse - ((2 * (frameCount - (period / 2)) * (maxPulse - minPulse))/ period)
    repeat localIndex from FIRST_SERVO_PIN to FIRST_SERVO_PIN + STATIC_SERVOS - 1
      Servo.Set(localIndex, servoPosition)                
    repeat localIndex from FIRST_OSCILLATING_SERVO_PIN to FIRST_OSCILLATING_SERVO_PIN + OSCILLATING_SERVOS - 1
      Servo.Set(localIndex, oscillatingPosition)
  
  
    if frameCount => period - 1
      frameCount := 0
    else
      frameCount++

    case ledMode  
      STATIC_MODE:   
        if servoPosition > CENTER
          ledShow := ((servoPosition - CENTER) >> 1) '<# $FF 
          outa[16..23] := ledShow
        else 'if globalPulse < CENTER
          ledShow := ((CENTER - servoPosition) >> 1) '<# $FF 
          outa[23..16] := ledShow
          
      OSCILLATION_MODE:
        ledShow := period >> 1
        outa[23..16] := ledShow
        
      ENDPOINT_LOW_MODE:
        if minPulse > CENTER
          ledShow := ((minPulse - CENTER) >> 1) '<# $FF 
          outa[16..23] := ledShow
        else 'if globalPulse < CENTER
          ledShow := ((CENTER - minPulse) >> 1) '<# $FF 
          outa[23..16] := ledShow
          
      ENDPOINT_HIGH_MODE:
        if maxPulse > CENTER
          ledShow := ((maxPulse - CENTER) >> 1) '<# $FF 
          outa[16..23] := ledShow
        else 'if globalPulse < CENTER
          ledShow := ((CENTER - maxPulse) >> 1) '<# $FF 
          outa[23..16] := ledShow

PUB DisplayHeading

  Debug.Clear       
  'Debug.str(string(13, "Debug Started "))
  Debug.str(string(13, "fileName = "))
  Debug.str(@fileName)
  Debug.char(13)
  Debug.str(string(16, "Servo Control"))
  Debug.str(string(13, "Use touchPads to change pulse length, end points, oscillation speed"))
  Debug.str(string(13, "P7 increase pulse length of high end point"))
  Debug.str(string(13, "P6 reduce pulse length of high end point"))
  Debug.str(string(13, "P5 increase pulse length of low end point"))
  Debug.str(string(13, "P4 reduce pulse length of low end point"))
  Debug.str(string(13, "P3 increase oscillation period (slow the oscillation)"))
  Debug.str(string(13, "P2 reduce oscillation period (speed up the oscillation)"))
  Debug.str(string(13, "P1 increase the pulse length of static servo"))
  Debug.str(string(13, "P0 reduce the pulse length of static servo")) 
  Debug.str(string(13, 13, "The static servos are on pins"))
  Debug.Dec(FIRST_SERVO_PIN)
  Debug.str(string(" through "))
  Debug.Dec(FIRST_SERVO_PIN + STATIC_SERVOS - 1)
  Debug.char(".")
  Debug.str(string(13, "The oscillatiing servos are on pins"))
  Debug.Dec(FIRST_OSCILLATING_SERVO_PIN)
  Debug.str(string(" through "))
  Debug.Dec(FIRST_OSCILLATING_SERVO_PIN + OSCILLATING_SERVOS - 1)
  Debug.char(".")
  Debug.char(13)
  Debug.char(13)
  Debug.ClearBelow
  
PUB DebugLoop | headingTime, headingInterval, differenceCount, localCount, localStatic, localOscillate, localMin, localMax, localPeriod  ' keep these together and in order
'' We'll use a seperate loop to send debug information
'' so it doesn't slow down the servo calculations.
'' It is likely that not all frames will be displayed
'' since the data can't be transmitted fast enough to
'' keep up with how quickly the seroPosition data is
'' changed.

  Debug.Start(DEBUG_BAUD)
  waitcnt(clkfreq / 20 + cnt)
  DisplayHeading
  
  localCount := period + 1
  ' not a legitamate frame so frameCount is sure not to be equal to it.
  headingInterval := clkfreq * 5 '10
  headingTime := cnt 
  repeat
    'if cnt - headingTime == headingInterval
    '  headingTime += headingInterval
    '  DisplayHeading
    waitcnt(clkfreq / 20 + cnt)
    repeat while frameCount == localCount
    differenceCount := 0
    repeat result from 0 to 20 step 4
      if long[@localCount + result] <> long[@frameCount + result]
        differenceCount++
    if differenceCount > 0    
      longmove(@localCount, @frameCount, 6) ' make sure all three data items correspond to the same frame.
      ' move the servoPosition values so they don't change while they
      ' are being displayed.
       
      Debug.str(string(Debug#PX, 1, "F# "))
      '**11h*Debug.DecPad(localCount, 6, " ")
      Debug.Dec(localCount)
      Debug.str(string(", frame= "))
      '**11h*Debug.DecPoint((frameInterval * 1000 )/ clkfreq, 1000)
      
      Debug.str(string(", MinP= "))
      '**11h*Debug.DecPad(minPulse, 4, " ")
      Debug.Dec(minPulse)
      Debug.str(string(", MaxP= "))
      '**11h*Debug.DecPad(maxPulse, 4, " ")
      Debug.Dec(maxPulse)
      Debug.str(string(", StaticP= "))
      '**11h*Debug.DecPad(servoPosition, 4, " ")
      Debug.Dec(servoPosition)
      Debug.str(string(", OscilP= "))
      '**11h*Debug.DecPad(oscillatingPosition, 4, " ")
      Debug.Dec(oscillatingPosition)
      Debug.str(string(", Period= "))
      '**11h*Debug.DecPad(period, 6, " ")
      Debug.Dec(period)
      Debug.str(string(" frames  "))
      'Debug.str(string(" frames or "))
      'Debug.DecPoint((period * frameInterval * 1000) / clkfreq, 1000 )
      'Debug.str(string(" seconds              "))
      {if period * frameInterval > clkfreq
        Debug.str(string(" or "))
        DecPoint(value, denominator)
        Debug.Dec(period * frameInterval * clkfreq)
        Debug.char(".")  
        Debug.Dec((period * frameInterval * (clkfreq / 1000))//1000)
        Debug.str(string(" s"))} 